<?php 

namespace Naiz\NaizFitSizeForm\Model;
use Naiz\NaizFitSizeForm\Api\FeedInterface;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SortOrder;
use Magento\Framework\Api\Filter;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Api\SortOrderBuilder;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Catalog\Model\ResourceModel\Category;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use \Magento\Framework\Webapi\Rest\Response;

class Feed implements FeedInterface {

  private $productRepository;
  private $searchCriteriaBuilder;
  private $sortOrderBuilder;
  private $categoryCollectionFactory;
  private $storeManager;
  private $request;
  private $response;
  private $jsonResultFactory;

  public function __construct(
      ProductRepositoryInterface $productRepository,
      SearchCriteriaBuilder $searchCriteriaBuilder,
      SortOrderBuilder $sortOrderBuilder,
      Category\CollectionFactory $categoryCollectionFactory,
      StoreManagerInterface $storeManager,
      RequestInterface $request,
      Response $response,
      JsonFactory $jsonResultFactory
  ) {
      $this->productRepository = $productRepository;
      $this->searchCriteriaBuilder = $searchCriteriaBuilder;
      $this->sortOrderBuilder = $sortOrderBuilder;
      $this->categoryCollectionFactory = $categoryCollectionFactory;
      $this->storeManager = $storeManager;
      $this->request = $request;
      $this->response = $response;
      $this->jsonResultFactory = $jsonResultFactory;
  }
  
  public function getFeed() {
    try {
      $count = $this->request->has("count") ? $this->request->getParam("count") : 200;
      $page = $this->request->has("page") ? $this->request->getParam("page") : 1;
      $type = $this->request->has("type") ? $this->request->getParam("type") : null;
      $last = $this->request->has("last") ? $this->request->getParam("last") : null;
      
      $searchCriteria = $this->searchCriteriaBuilder->create();
      $filterGroups = array();
      if ($type !== null) {
        $filter = new Filter();
        $filter->setField("type_id")
               ->setValue($type)
               ->setConditionType("eq");
        $filterGroup = new FilterGroup();
        $filterGroup->setFilters(array($filter));
        $filterGroups[] = $filterGroup;
      }
      if ($last !== null) {
        $ago = new \DateTime((intval($last) ?: 7) . " days ago midnight");

        $filter = new Filter();
        $filter->setField("updated_at")
               ->setValue($ago->format('Y-m-d H:i:s'))
               ->setConditionType("gteq");
        $filterGroup = new FilterGroup();
        $filterGroup->setFilters(array($filter));
        $filterGroups[] = $filterGroup;
      }
      $searchCriteria->setFilterGroups($filterGroups);
      $searchCriteria->setPageSize($count)->setCurrentPage($page);
      $productsResult = $this->productRepository->getList($searchCriteria);
      $products = $productsResult->getItems();
      if ($productsResult->getTotalCount() > count($products) + ($count * ($page - 1))) {
        $pathInfo = parse_url((isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' ? 'https' : 'http') . '://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
        $queryString = array_key_exists('query', $pathInfo) ? $pathInfo['query'] : "";
        parse_str($queryString, $queryArray);
        $queryArray['page'] = array_key_exists("page", $queryArray) && $queryArray['page'] ? $queryArray['page'] + 1 : 2;
        $newQueryStr = http_build_query($queryArray);
        $next = $pathInfo['scheme'] . "://" . $pathInfo['host'] . $pathInfo['path'] .'?'.$newQueryStr;
        $this->response->setHeader('link', "<" . $next . ">; rel=\"next\"" , true);
      }
      $stores2 = $this->storeManager->getStores();
      $stores = array();
      foreach ($stores2 as $store) {
        if ($store->IsActive()) $stores[] = $store;
      }
      $data = array_map(function($product) use ($stores) {
        $banned_attributes = array ("meta_description","description","page_layout","custom_design","custom_layout","gift_message_available","options_container","price_view","media_gallery","ts_packaging_type","ts_packaging_id","quantity_and_stock_status","shipment_type","msrp_display_actual_price_type","visibility");

        $categoriesFac = $this->categoryCollectionFactory->create();
        $categoriesFac->addAttributeToSelect('*');        
        $categoryIds = $product->getCategoryIds();
        $categories = $categoryIds ? $categoriesFac->addAttributeToFilter('entity_id', $categoryIds) : $categoriesFac;  

        $options = $product->getTypeId() == "configurable" ? array_merge(... $product->getTypeInstance()->getConfigurableOptions($product)) : array();
        $options = array_reduce(array_map(function($option) {
          return array("code" => $option["attribute_code"], "title" => $option["option_title"]);
        }, $options), function($carry, $item) { 
          if(!in_array($item, $carry))
          $carry[] = $item;
          return $carry;
        }, array());
        
        $attributes = $product->getAttributes();
        $attributes2 = array();
        foreach ($attributes as $attribute){
          $value = $attribute->getFrontend()->getValue($product);
          $code = $attribute->getAttributeCode();
          if ($value && !in_array($code,$banned_attributes)) $attributes2[$code] = $value;
        }
        

        $categories = count($categoryIds) > 0 ? $categories->getItems() : array();
        $categories2 = array();
        foreach ($categories as $category) {
          $categories2[] = array ( "code" => $category->getId(), "parent_code" => $category->getParentId(), "path" => $category->getPath(), "position" => $category->getPosition(), "level" => $category->getLevel(), "name" => $category->getName(), "active" => $category->getIsActive(),"url_key"=>$category->getUrlKey(),"url_path"=>$category->getUrlPath());
        }

        $urls = array();
        foreach ($stores as $store){
          $urls[] = $product->setStoreId($store->getId())->getUrlModel()->getUrlInStore($product);
        }

        $price = $product->getTypeId() == "configurable" ? $product->getPriceInfo()->getPrice('regular_price')->getMaxRegularAmount()->getValue() : $product->getPrice();
        return array(
          "name" => $product->getName(),
          "sku" => $product->getSku(),
          "url" => $product->getUrlModel()->getUrl($product),
          "other_urls" => $urls,
          "price" => $product->getPrice(),
          "status" => $product->getStatus(),
          "categoryIds" => $product->getCategoryIds(),
          "categories" => $categories2,
          "qty" => $product->getQty(),
          "price" => ($price),
          "inStock" => $product->isInStock(),
          "images" => array_map(function($image) {return $image->getUrl();}, $product->getMediaGalleryImages() ? $product->getMediaGalleryImages()->getItems() : array()),
          "options" => $options,
          "attributes" => $attributes2
        );
      }, $products);
      return $data;
    } catch (Exception $e) {
      return array("error" => "ERROR: " . $e->getMessage());
    } catch (\Throwable $t) {
      return array("error" => "NULL ERROR: ". $t->getMessage());
    }
	}
}