<?php

namespace Naiz\NaizFitSizeForm\Observer;

use Magento\Framework\Event\ObserverInterface;
use Psr\Log\LoggerInterface;
use Naiz\NaizFitSizeForm\Helper\Data;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Directory\Model\CountryFactory;
use Magento\Framework\UrlInterface;
use Magento\Framework\HTTP\Client\Curl;

class GetOrderData implements ObserverInterface {

  protected $logger;
  protected $_countryFactory;
  protected $helper;
  protected $storeManager;
  protected $_curl;

  public function __construct(
    LoggerInterface $logger,
    CountryFactory $countryFactory,
    StoreManagerInterface $storeManager,
    Curl $curl,
    Data $helper
  ) {
    $this->logger = $logger;
    $this->_countryFactory = $countryFactory;
    $this->helper = $helper;
    $this->storeManager = $storeManager;
    $this->_curl = $curl;
  }

  public function execute(\Magento\Framework\Event\Observer $observer)
  {
    try {
        $url = ($this->helper->getApiEndpoint() ?: "https://backend.production.naiz.fit/api/partners/hook/") . parse_url($this->storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_WEB), PHP_URL_HOST) . "/" . $this->helper->getPartnerKey();
        $order = $observer->getEvent()->getOrder();

        $quoteId = $order->getQuoteId();
      
        $billingAddress = $order->getBillingAddress();
        $billingCountry = $this->_countryFactory->create()->loadByCode($billingAddress->getCountryId());
        $shippingAddress = $order->getShippingAddress();
        $shippingCountry = $this->_countryFactory->create()->loadByCode($shippingAddress->getCountryId());
        
        $items = array_map(function($item) {
          $productOptions = $item->getProductOptions();
          $options = array_map(function($option) {
            return $option["value"];
          }, $productOptions && array_key_exists("attributes_info", $productOptions) ? ($productOptions["attributes_info"] ?: []) : []);
        
          return array(
            "name" => $item->getName(),
            "price" => $item->getPrice(),
            "qty" => $item->getQtyOrdered(),
            "total" => $item->getRowTotal(),
            "sku" => $item->getSku(),
            "product_sku" => $item->getProduct()->getSku(),
            "options" => implode("-", $options),
            "discount" => $item->getDiscountAmount(),
            "id" => $item->getItemId()
          );
        }, $order->getAllVisibleItems() ?: []);

        $post = array(
          //"endpoint" => $url,
          "quoteId" => $quoteId,
          "fullfillment_date" => $order->getCreatedAt(),
          "partner_hash" => $this->helper->getPartnerKey(),
          "domain" => $this->storeManager->getStore()->getBaseUrl(UrlInterface::URL_TYPE_WEB),
          "internal_ref" => $order->getIncrementId(),
          "status" => $order->getStatus(),
          "currency" => $order->getOrderCurrencyCode(),
          "grand_total" => $order->getGrandTotal(),
          "sub_total" => $order->getSubTotal(),
          "sub_total_with_tax" => $order->getSubtotalInclTax(),
          "shipping_with_tax" => $order->getBaseShippingInclTax(),
          "base_sub_total" => $order->getBaseSubtotal(),
          "base_subtotal_with_tax" => $order->getBaseSubtotalInclTax(),
          "discount_amount" => $order->getDiscountAmount(),
          "discount_tax_compensation_amount" => $order->getDiscountTaxCompensationAmount(),
          "taxes" => $order->getTaxAmount(),
          "items" => $items
        );
        $result = $this->_curl->post($url, json_encode($post));
        $this->logger->info("NaizFit Size Form: Transaction '" . $order->getIncrementId() . "' registered -> " . var_export($this->_curl->getBody(), true));
        //$this->logger->info(var_export($result, true));
    } catch (Exception $e) {
        $this->logger->error("NaizFit Size Form: ".$e->getMessage());
    }
  }
}